# coding: utf-8
from unittest import TestCase
from lxml.html import fromstring

from weblib.content import find_content_blocks

class ContentTestCase(TestCase):
    def test_find_content_blocks(self):
        porno = u'порно ' * 100
        redis = u'редис ' * 100
        html = ('<div>%s</div><p>%s' % (porno, redis))
        tree = fromstring(html)
        blocks = list(find_content_blocks(tree, min_length=100))
        #print '>>>'
        #print blocks[0]
        #print len(blocks[0])
        #print '<<<'
        #print ')))'
        #print porno.strip()
        #print len(porno.strip())
        #print '((('
        self.assertEqual(blocks[0], porno)
        self.assertEqual(blocks[1], redis)



import warnings
import logging


# WTF Grab doing here?
def warn(msg):
    warnings.warn(msg, category=GrabDeprecationWarning, stacklevel=3)


# ***********************************
# Base Classes to build Custom Errors
# ***********************************

class WeblibError(Exception):
    """
    Base class for all custom exceptions
    defined in weblib package.
    """

class ResponseNotValid(WeblibError):
    pass


# ********************
# Errors inside weblib
# ********************

class RuntimeConfigError(WeblibError):
    """
    Raised when passed parameters do not makes sense
    or conflict with something.
    """


# **********************
# Data Processing Errors
# **********************

class DataNotFound(WeblibError, IndexError):
    """
    Raised when it is not possible to find requested
    data.
    """


# *****************************
# ResponseNotValid based Classes
# *****************************

class DataNotValid(ResponseNotValid):
    pass


class RequestBanned(ResponseNotValid):
    pass


class CaptchaRequired(ResponseNotValid):
    pass


class PageNotFound(ResponseNotValid):
    pass


class AccessDenied(ResponseNotValid):
    pass


# *****************************
# ResponseNotValid based Classes
# specific to HTTP code errors
# *****************************

class HttpCodeNotValid(ResponseNotValid): 
    pass


class HttpCodeZero(HttpCodeNotValid):
    pass


# ********************
# Other Errors
# ********************
class NextPageNotFound(WeblibError):
    pass



# **********
# Deprecated
# **********
class CaptchaError(CaptchaRequired):
    def __init__(self, *args, **kwargs):
        logging.error('Class CaptchaError is deprecated. Use CaptchaRequired.')
        super(CaptchaError, self).__init__(*args, **kwargs)


class BanError(RequestBanned):
    def __init__(self, *args, **kwargs):
        logging.error('Class BanError is deprecated. Use RequestBanned.')
        super(BanError, self).__init__(*args, **kwargs)


class UnexpectedData(DataNotValid):
    def __init__(self, *args, **kwargs):
        logging.error('Class UnexpectedData is deprecated. Use DataNotValid.')
        super(UnexpectedData, self).__init__(*args, **kwargs)


class HttpCodeUnexpected(HttpCodeNotValid):
    def __init__(self, *args, **kwargs):
        logging.error('Class HttpCodeUnexpected is deprecated. '
                      'Use HttpCodeNotValid.')
        super(HttpCodeUnexpected, self).__init__(*args, **kwargs)


class UnexpectedContent(DataNotValid):
    def __init__(self, *args, **kwargs):
        logging.error('Class UnexpectedContent is deprecated. '
                      'Use DataNotValid.')
        super(UnexpectedContent, self).__init__(*args, **kwargs)



from urllib.request import quote, unquote
import re

from weblib.error import RequestBanned, DataNotValid, HttpCodeZero

ALLOWED_HTTP_CODES = (200,)


def build_search_url(query, lr=84):
    # https://search.yaca.yandex.ru/geo.c2n
    # 84 - USA
    # 111 - europe
    # 166 - xUSSR
    # 318 - universal
    # 225 - russia
    url_tpl = 'http://yandex.ru/search/?text=%s&lr=%d'
    url = url_tpl % (quote(query), lr)
    return url


def check_integrity(grab):
    #if grab.doc.code == -1: # FIX
    #    raise RequestBanned('Ban (HTTP code %d)' % grab.doc.code)
    if grab.doc.select('//img[contains(@src, "/captchaimg?")]').exists():
        raise RequestBanned('Ban (captcha)')
    elif grab.doc.code != 200:
        raise DataNotValid('Non-200 HTTP code: %d' % grab.doc.code)


def check_search_result_integrity(grab):
    check_integrity(grab)
    if not grab.doc('//input[@name="text"]').exists():
        raise DataNotValid('Expected HTML element not found')


def check_cache_integrity(grab):
    check_integrity(grab)
    if not grab.doc('//script[contains(@src,'
                    '"yandex.st/hilitedaemon-js")]').exists():
        raise DataNotValid('Expected yandex.st script not found')


def parse_search_result(grab):
    res = []
    for elem in grab.doc('//h2[@class="serp-item__title"]'):
        res.append({
            'url': elem.select('a/@href').text(),
            'anchor': elem.select('a').text(),
            'cache_url': elem.select(
                '..//a[contains(@href, "hghltd.yandex.net")]'
                '/@href').text(default=None),
        })
    return res



# coding: utf-8
from unittest import TestCase

from weblib.russian import slugify, parse_ru_month

class ToolsRussianTestCase(TestCase):
    def test_parse_ru_month(self):
        self.assertEqual(1, parse_ru_month(u'Январь'))
        self.assertEqual(1, parse_ru_month(u'Января'))
        self.assertEqual(1, parse_ru_month(u'январь'))
        self.assertEqual(1, parse_ru_month(u'января'))
        self.assertEqual(1, parse_ru_month(u'яНВарЬ'))
        self.assertEqual(12, parse_ru_month(u'деКАБрЯ'))

    def test_slugify_transliteration(self):
        self.assertEqual('fu', slugify('фу'))
        self.assertEqual('fu', slugify('фу#'))
        self.assertEqual('fu-bin', slugify('фу#bin'))
        self.assertEqual('bin', slugify('####bin'))
        self.assertEqual('bin', slugify('####бин'))

    def test_slugify_dot_processing(self):
        self.assertEqual('bi-n', slugify('####би.н'))
        self.assertEqual('bi.n', slugify('####би.н', dot_allowed=True))

    def test_slugify_limit(self):
        self.assertEqual('bi-n', slugify('####би.н'))
        self.assertEqual('bi-n', slugify('####би.н', limit=100))
        self.assertEqual('bi-n', slugify('####би.н', limit=4))
        self.assertEqual('bi-', slugify('####би.н', limit=3))

    def test_slugify_lower(self):
        # by default lower option is True
        self.assertEqual('bi-n', slugify('####БИ.н'))

        self.assertEqual('BI-n', slugify('####БИ.н', lower=False))

        # test also english letters
        self.assertEqual('bi-n', slugify('####bi.н'))
        self.assertEqual('BI-n', slugify('####BI.н', lower=False))

    def test_slugify_default(self):
        self.assertEqual('', slugify('####'))
        self.assertEqual('z', slugify('####', default='z'))
        self.assertEqual(None, slugify('####', default=None))



# coding: utf-8
from unittest import TestCase
import six

from weblib.html import find_refresh_url, decode_entities


class HtmlTestCase(TestCase):

    def test_find_refresh_url(self):
        url = find_refresh_url("""
            <meta http-equiv="refresh" content="5">
        """)
        self.assertEqual(None, url)

        url = find_refresh_url("""
            <meta http-equiv="refresh" content="5;URL='http://example.com/'">
        """)
        self.assertEqual('http://example.com/', url)

        url = find_refresh_url("""
            <meta http-equiv="refresh" content="0;URL='http://example.com/'">
        """)
        self.assertEqual('http://example.com/', url)

        url = find_refresh_url("""
            <meta http-equiv="refresh" content="5; url=http://example.com/">
        """)
        self.assertEqual('http://example.com/', url)

        url = find_refresh_url("""
            <meta http-equiv="refresh" content=" 0 ; url=http://example.com/">
        """)
        self.assertEqual('http://example.com/', url)

    def test_decode_entities(self):
        html = u'&#8594;'
        self.assertEquals(six.unichr(8594), decode_entities(html))

        html = u'&rarr;'
        self.assertEquals(six.unichr(8594), decode_entities(html))

        html = u'&#x2192;'
        self.assertEquals(six.unichr(8594), decode_entities(html))



# coding: utf-8
GOOGLE_HOST_LIST = """
google.ac
google.ad
google.ae
google.al
google.am
google.as
google.at
google.az
google.ba
google.be
google.bf
google.bg
google.bi
google.bj
google.bs
google.bt
google.by
google.ca
google.cat
google.cc
google.cd
google.cf
google.cg
google.ch
google.ci
google.cl
google.cm
google.co.ao
google.co.bw
google.co.ck
google.co.cr
google.co.id
google.co.il
google.co.in
google.co.jp
google.co.ke
google.co.kr
google.co.ls
google.co.ma
google.co.mz
google.co.nz
google.co.th
google.co.tz
google.co.ug
google.co.uk
google.co.uz
google.co.ve
google.co.vi
google.co.za
google.co.zm
google.co.zw
google.com
google.com.af
google.com.ag
google.com.ai
google.com.ar
google.com.au
google.com.bd
google.com.bh
google.com.bn
google.com.bo
google.com.br
google.com.bz
google.com.co
google.com.cu
google.com.cy
google.com.do
google.com.ec
google.com.eg
google.com.et
google.com.fj
google.com.gh
google.com.gi
google.com.gt
google.com.hk
google.com.jm
google.com.kh
google.com.kw
google.com.lb
google.com.ly
google.com.mm
google.com.mt
google.com.mx
google.com.my
google.com.na
google.com.ng
google.com.ni
google.com.np
google.com.om
google.com.pa
google.com.pe
google.com.pg
google.com.ph
google.com.pk
google.com.pr
google.com.py
google.com.qa
google.com.sa
google.com.sb
google.com.sg
google.com.sl
google.com.sv
google.com.tj
google.com.tn
google.com.tr
google.com.tw
google.com.ua
google.com.uy
google.com.vc
google.com.vn
google.cv
google.cz
google.dj
google.dk
google.dm
google.dz
google.ee
google.es
google.fi
google.fm
google.fr
google.ga
google.ge
google.gg
google.gl
google.gm
google.gp
google.gr
google.gy
google.hn
google.ht
google.hu
google.ie
google.im
google.iq
google.is
google.it
google.je
google.jo
google.kg
google.ki
google.kz
google.la
google.li
google.lk
google.lt
google.lu
google.lv
google.md
google.me
google.mg
google.mk
google.ml
google.mn
google.ms
google.mu
google.mv
google.mw
google.ne
google.nl
google.no
google.nr
google.nu
google.pl
google.pn
google.ps
google.pt
google.ro
google.rs
google.ru
google.rw
google.sc
google.se
google.sh
google.si
google.sk
google.sm
google.sn
google.so
google.st
google.td
google.tg
google.tk
google.tl
google.tm
google.tn
google.to
google.tt
google.us
google.vg
google.vu
google.ws
""".strip().splitlines()



# coding: utf-8
from unittest import TestCase
import six
from codecs import open
from weblib.encoding import make_str
from weblib.export.mysql_dumper import MysqlCSVDumper, build_import_sql
from tempfile import mkstemp
import os

IMPORT_SQL = r'''LOAD DATA LOCAL INFILE "/tmp/x.sql"
REPLACE INTO TABLE foo
character set utf8
fields terminated by "," optionally enclosed by '"'
lines terminated by "\r\n"
(col1,col2);'''

class MysqlCSVDumperTestCase(TestCase):
    def test_constructor(self):
        fh, path = mkstemp()
        MysqlCSVDumper(path)
        os.unlink(path)

    def test_normalize_none_value(self):
        fh, path = mkstemp()
        dumper = MysqlCSVDumper(path)
        dumper.add_row((None,))
        dumper.close()
        self.assertTrue(r'\N' in open(path, encoding='utf-8').read())
        os.unlink(path)

    def test_normalize_unicode(self):
        fh, path = mkstemp()
        dumper = MysqlCSVDumper(path)
        dumper.add_row((u'фуу',))
        dumper.close()
        self.assertTrue(u'фуу' in open(path, encoding='utf-8').read())
        os.unlink(path)

    def test_normalize_bytes(self):
        fh, path = mkstemp()
        dumper = MysqlCSVDumper(path)
        dumper.add_row((make_str('фуу'),))
        dumper.close()
        self.assertTrue(u'фуу' in open(path, encoding='utf-8').read())
        os.unlink(path)

    def test_normalize_int(self):
        fh, path = mkstemp()
        dumper = MysqlCSVDumper(path)
        dumper.add_row((1,))
        dumper.close()
        self.assertTrue(u'1' in open(path, encoding='utf-8').read())
        os.unlink(path)

    def test_build_import_sql(self):
        self.assertEqual(
            IMPORT_SQL, build_import_sql('/tmp/x.sql', 'foo', ['col1', 'col2']))



from urllib.request import quote, unquote

from weblib.error import RequestBanned, DataNotValid, HttpCodeZero

ALLOWED_HTTP_CODES = (999, 200)


def build_search_url(query, cc='us'):
    url_tpl = 'http://www.bing.com/search?q=%s&cc=%s'
    url = url_tpl % (quote(query), cc)
    return url


def check_integrity(grab):
    #if grab.doc.code == 999:
    #    raise RequestBanned('Ban (HTTP code %d)' % grab.doc.code)
    if grab.doc.code != 200:
        raise HttpCodeNotValid('Non-200 HTTP code: %d' % grab.doc.code)


def check_search_result_integrity(grab):
    check_integrity(grab)
    if not grab.doc('//input[@name="q"]').exists():
        raise DataNotValid('Search query input not found')


def check_cache_integrity(grab):
    check_integrity(grab)
    if not grab.doc('//div[@class="b_vPanel"]').exists():
        raise DataNotValid('Div[@class="b_vPanel"] not found')


def parse_search_result(grab):
    res = []
    for elem in grab.doc('//ol[@id="b_results"]/li[contains(@class, "b_algo")'
                         ' and .//h2/a]'):
        link = elem.select('.//h2/a')
        try:
            b_attr = elem.select('.//div[@class="b_attribution"]/@u')\
                         .text(default=None)
            if b_attr:
                x, y, attr_d, attr_w = b_attr.split('|')
                cache_url = 'http://cc.bingj.com/cache.aspx?q=zzz&d=%s&w=%s' % (
                    attr_d, attr_w)
            else:
                cache_url = None
        except Exception as ex:
            logging.error(exc_info=ex)
            import pdb; pdb.set_trace()
        res.append({
            'url': link.attr('href'),
            'anchor': link.text(),
            'cache_url': cache_url,
        })
    return res



from urllib.request import quote, unquote

from weblib.error import RequestBanned, DataNotValid, HttpCodeZero

ALLOWED_HTTP_CODES = (999, 200)


def build_search_url(query):
    url_tpl = 'https://search.yahoo.com/search?p=%s&ei=UTF-8&nojs=1'
    url = url_tpl % quote(query)
    return url


def check_integrity(grab):
    if grab.doc.code == 999:
        raise RequestBanned('Ban (HTTP code %d)' % grab.doc.code)
    elif grab.doc.code != 200:
        raise HttpCodeNotValid('Non-200 HTTP code: %d' % grab.doc.code)


def check_search_result_integrity(grab):
    check_integrity(grab)
    if not grab.doc('//input[@name="p"]').exists():
        raise DataNotValid('Search query input not found')


def check_cache_integrity(grab):
    check_integrity(grab)
    if not grab.doc('//div[@class="cacheContent"]').exists():
        raise DataNotValid('Div[@class="cacheContent"] not found')


def extract_encoded_url(url):
    return unquote(url.split('/RU=')[1].split('/')[0])


def parse_search_result(grab):
    res = []
    for elem in grab.doc('//ol/li//div[h3[@class="title"]/a]'):
        data = elem.select('h3/a/@href').text().strip()
        if '/RU=' in data:
            url = extract_encoded_url(data)
        else:
            url = data

        cache_url = None
        for node in elem.select('..//a[@href]'):
            node_url = node.attr('href')
            if '/RU=' in node_url:
                node_url = extract_encoded_url(node_url)
            if '/srpcache' in node_url:
                cache_url = node_url
                break

        res.append({
            'url': url,
            'anchor': elem.select('h3/a').text().strip(),
            'cache_url': cache_url,
        })
    return res



import sys 
import logging

logger = logging.getLogger('weblib.progress')


class Progress(object):
    def __init__(self, step=None, total=None, stop=None, name='items', level=logging.DEBUG):
        if total is None and step is None:
            raise Exception('Both step and total arguments are None')
        if total and not step:
            step = int(total / 20) 
        if step == 0:
            step = total
        self.step = step
        self.count = 0 
        self.total = total
        self.stop = stop
        self.name = name
        self.logging_level = level

    def tick(self):
        self.count += 1
        if not self.count % self.step:
            if self.total:
                percents = ' [%d%%]' % int((self.count / float(self.total)) * 100)
            else:
                percents = ''
            logger.log(self.logging_level, 'Processed %d %s%s' % (self.count, self.name, percents))
        if self.count == self.stop:
            logger.log(self.logging_level, 'Reached stop value %d' % self.stop)
            sys.exit()



from random import choice
from urllib.request import quote

from weblib.scraper.google_host import GOOGLE_HOST_LIST
from weblib.error import DataNotValid, RequestBanned, NextPageNotFound

__all__ = ['build_search_url', 'check_integrity',
           'parse_search_result', 'parse_next_page',
           'ALLOWED_HTTP_CODES']

ALLOWED_HTTP_CODES = (403, 503, 200)


def build_search_url(query, host=None, country='us'):
    if host is None:
        host = choice(GOOGLE_HOST_LIST)
    url_tpl = 'https://www.%s/search?q=%s&gl=%s'
    return url_tpl % (host, quote(query), country)


def check_integrity(grab):
    if grab.doc.code in (503, 403) or grab.doc('//input[@name="captcha"]').exists():
        raise RequestBanned('Captcha found')


def check_search_result_integrity(grab):
    check_integrity(grab)
    if not grab.doc('//div[@id="res"]').exists():
        raise DataNotValid('Content of response has unexpected format.')


def check_cache_integrity(grab):
    check_integrity(grab)
    if grab.doc.code == 404:
        pass
    elif not grab.doc('//div[@id="google-cache-hdr"]').exists():
        raise DataNotValid('Google Cache Header not found')


def parse_search_result(grab):
    res = []
    for elem in grab.doc('//h3[following-sibling::div[@class="s"]]'):
        res.append({
            'url': elem.select('a/@href').text().strip(),
            'anchor': elem.select('a').text().strip(),
            #snippet = elem.select('./following-sibling::div[@class="s"]')
        })
    return res


def parse_next_page(grab):
    """
    Returns tuple (next_page_number, next_page_url)
    """
    elem = grab.doc('//div[@id="navcnt"]')
    cur_page = int(elem.select('.//td[@class="cur"]').text())
    pages = {}
    for sub_elem in elem.select('.//td/a[@class="fl"]'):
        number = int(sub_elem.text())
        pages[number] = sub_elem.attr('href')
    next_page = cur_page + 1
    if next_page in pages:
        return next_page, pages[next_page]
    else:
        raise NextPageNotFound



from setuptools import setup, find_packages
import os

ROOT = os.path.dirname(os.path.realpath(__file__))


setup(
    name = 'weblib',
    version = '0.1.15',
    description = 'Set of tools for web scraping projects',
    author = 'Gregory Petukhov',
    author_email = 'lorien@lorien.name',
    install_requires = ['pytils', 'six', 'lxml'],
    packages = find_packages(exclude=['test']),
    license = "MIT",
    classifiers = (
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ),
)



# coding: utf-8
from unittest import TestCase
from six.moves.urllib.parse import quote

from weblib.http import normalize_url, RESERVED_CHARS

class HttpTestCase(TestCase):
    def test_normalize_url_idn(self):
        url = 'http://почта.рф/path?arg=val'
        norm_url = 'http://xn--80a1acny.xn--p1ai/path?arg=val'
        self.assertEqual(norm_url, normalize_url(url))

    def test_normalize_url_unicode_path(self):
        url = u'https://ru.wikipedia.org/wiki/Россия'
        norm_url = 'https://ru.wikipedia.org/wiki'\
                   '/%D0%A0%D0%BE%D1%81%D1%81%D0%B8%D1%8F'
        self.assertEqual(norm_url, normalize_url(url))

    def test_normalize_url_unicode_query(self):
        url = 'https://ru.wikipedia.org/w/index.php?title=Заглавная_страница'
        norm_url = 'https://ru.wikipedia.org/w/index.php'\
                   '?title=%D0%97%D0%B0%D0%B3%D0%BB%D0%B0%D0%B2%D0%BD'\
                   '%D0%B0%D1%8F_%D1%81%D1%82%D1%80'\
                   '%D0%B0%D0%BD%D0%B8%D1%86%D0%B0'
        self.assertEqual(norm_url, normalize_url(url))

    def test_normalize_url_with_mix_of_norm_and_unnorm(self):
        url = 'http://test.com/!?%21'
        norm_url = 'http://test.com/!?%21'
        self.assertEqual(norm_url, normalize_url(url))

    def test_normalize_url_normalized_ascii(self):
        url = 'http://test.com/%21?%21'
        norm_url = 'http://test.com/%21?%21'
        self.assertEqual(norm_url, normalize_url(url))


    def test_normalize_normalized_non_ascii(self):
        url = 'http://www.film.ru/movies/a-z/%D0%9F?%d0%9f'
        norm_url = 'http://www.film.ru/movies/a-z/%D0%9F?%d0%9f'
        self.assertEqual(norm_url, normalize_url(url))

    def test_normalize_non_quoted_percent(self):
        url = 'http://test.com/%9z%21'
        norm_url = 'http://test.com/%9z%21'
        self.assertEqual(norm_url, normalize_url(url))

    def test_quoted_query_in_query(self):
        url = (u"https://graph.facebook.com/fql?q=SELECT%20url%20,total_count"
               u"%20FROM%20link_stat%20WHERE%20url%20in%20('http%3A%2F%2F"
               u"www.ksl.com%2F%3Fsid%3D34840696%26nid%3D148')")
        #norm_url = ("https://graph.facebook.com/fql?q=SELECT%20url%20%2C"
        #            "total_count%20FROM%20link_stat%20WHERE%20url%20in"
        #            "%20%28%27http%3A%2F%2Fwww.ksl.com%2F%3Fsid%3D"
        #            "34840696%26nid%3D148%27%29")
        self.assertEqual(url, normalize_url(url))

    def test_reserved_delims(self):
        path = RESERVED_CHARS.replace('/', '')
        query = RESERVED_CHARS.replace('?', '')
        fragment = RESERVED_CHARS.replace('#', '')
        url = 'http://domain.com/%s?%s#%s' % (path, query, fragment)
        self.assertEqual(url, normalize_url(url))
